"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
exports.__esModule = true;
exports.withDestroyable = exports.CacheStrategy = void 0;
var iterators_1 = require("../iterators");
var utils_1 = require("../utils");
var utils_2 = require("../utils");
var value_1 = require("../value");
/**
 * Describes some strategy holding up to `capacity` items at the same moment.
 */
var CacheStrategy = /** @class */ (function (_super) {
    __extends(CacheStrategy, _super);
    function CacheStrategy(capacity) {
        return _super.call(this, capacity) || this;
    }
    /**
     * Removes all items from the strategy.
     *
     */
    CacheStrategy.prototype.clear = function () {
        while (!this.isEmpty()) {
            this.take();
        }
    };
    /**
     * Reserves place for a new item.
     * @param value
     */
    CacheStrategy.prototype.reservePlace = function () {
        if (this.willBeFull()) {
            var removedItem = this.take();
            if (removedItem !== value_1.NO_VALUE) {
                return utils_1.Result.removed((0, iterators_1.once)(removedItem));
            }
        }
        return utils_1.Result.empty();
    };
    return CacheStrategy;
}(utils_2.HasCapacity));
exports.CacheStrategy = CacheStrategy;
/**
 * Forces provided strategy class to call `.destroy` on removed entities.
 */
function withDestroyable(strategy) {
    var WithDestroyable = /** @class */ (function (_super) {
        __extends(WithDestroyable, _super);
        function WithDestroyable() {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            return _super.apply(this, __spreadArray([], __read(args), false)) || this;
        }
        WithDestroyable.prototype.drop = function (value) {
            var dropped = _super.prototype.drop.call(this, value);
            if (dropped) {
                value.destroy();
            }
            return dropped;
        };
        WithDestroyable.prototype.take = function () {
            var item = _super.prototype.take.call(this);
            if (item !== value_1.NO_VALUE) {
                item.destroy();
            }
            return item;
        };
        return WithDestroyable;
    }(strategy));
    return WithDestroyable;
}
exports.withDestroyable = withDestroyable;
//# sourceMappingURL=CacheStrategy.js.map