"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
exports.__esModule = true;
exports.MultiKeyQueue = void 0;
var LinkedList_1 = require("./LinkedList");
var types_1 = require("./types");
var value_1 = require("../value");
var iterators_1 = require("../iterators");
var storage_1 = require("../storage");
/**
 * An indexed queue where each item is an indexed queue of keys.
 * An item itself should implement `IndexedOrderedCollection`.
 */
var MultiKeyQueue = /** @class */ (function (_super) {
    __extends(MultiKeyQueue, _super);
    function MultiKeyQueue(values) {
        if (values === void 0) { values = iterators_1.EMPTY_ITERABLE; }
        var _this = _super.call(this) || this;
        _this.list = new LinkedList_1.LinkedList();
        _this.map = new storage_1.SafeMapStorage();
        (0, iterators_1.forEach)(function (value) { return _this.pushBack(value); }, values);
        return _this;
    }
    /**
     * Returns amount of keys (references) stored in a map.
     *
     */
    MultiKeyQueue.prototype.len = function () {
        return this.map.len();
    };
    /**
     * Drops an item associated with the supplied key.
     * Returns dropped value in case of a successful drop or `NO_VALUE` if the value wasn't found.
     * @param key
     *
     */
    MultiKeyQueue.prototype.drop = function (key) {
        var maybeNoValueListNode = this.map.get(key);
        if (maybeNoValueListNode === value_1.NO_VALUE) {
            return value_1.NO_VALUE;
        }
        var listNode = maybeNoValueListNode;
        if (!this.remove(listNode)) {
            throw new Error("Inconsistency");
        }
        return listNode.value;
    };
    /**
     * Moves node to the front of the queue.
     * Returns `true` in case of success.
     * @param listNode
     *
     */
    MultiKeyQueue.prototype.moveFront = function (listNode) {
        return this.list.moveFront(listNode);
    };
    /**
     * Moves node to the back of the queue.
     * Returns `true` in case of success.
     * @param listNode
     *
     */
    MultiKeyQueue.prototype.moveBack = function (listNode) {
        return this.list.moveBack(listNode);
    };
    /**
     * Removes an item from the queue.
     * Returns `true` in case of success.
     * @param listNode
     *
     */
    MultiKeyQueue.prototype.remove = function (listNode) {
        var removed = this.list.remove(listNode);
        if (removed) {
            this.dissocKeys(listNode.value);
        }
        return removed;
    };
    /**
     * Returns `true` if supplied item belongs to the collection.
     */
    MultiKeyQueue.prototype.contains = function (listNode) {
        return this.list.contains(listNode);
    };
    /**
     * Drops supplied key from the map.
     * Item belonging to this key will be deleted only if it has no more references in the map.
     * Returns `true` in case of a successful removal or `false` if the value wasn't found.
     * @param key
     *
     */
    MultiKeyQueue.prototype.dropKey = function (key) {
        var maybeNoValueListNode = this.map.get(key);
        if (maybeNoValueListNode === value_1.NO_VALUE) {
            return false;
        }
        var listNode = maybeNoValueListNode;
        this.map.drop(key);
        if (!listNode.value.dropKey(key)) {
            throw new Error("Inconsistency");
        }
        if (listNode.value.isEmpty()) {
            this.list.remove(listNode);
        }
        return true;
    };
    /**
     * Inserts given value after the supplied raw linked list node.
     * @param node
     * @param key
     *
     */
    MultiKeyQueue.prototype.insertAfter = function (node, value) {
        var listNode = this.list.insertAfter(node, value);
        if (listNode == null) {
            return value_1.NO_VALUE;
        }
        this.assocKeys(value, listNode);
        return listNode;
    };
    /**
     * Inserts given value before the supplied raw linked list node.
     * @param node
     * @param key
     *
     */
    MultiKeyQueue.prototype.insertBefore = function (node, value) {
        var listNode = this.list.insertBefore(node, value);
        if (listNode == null) {
            return value_1.NO_VALUE;
        }
        this.assocKeys(value, listNode);
        return listNode;
    };
    /**
     * Returns value associated with the given key or `NO_VALUE` if the value wasn't found.
     * @param key
     *
     */
    MultiKeyQueue.prototype.get = function (key) {
        return this.map.get(key);
    };
    /**
     * Returns `true` if value associated with the given key exists.
     * @param key
     *
     */
    MultiKeyQueue.prototype.has = function (node) {
        return this.map.has(node);
    };
    /**
     * Adds a key for the supplied item to the beginning of its queue.
     * @param key
     * @param listNode
     *
     */
    MultiKeyQueue.prototype.addKeyFront = function (key, item) {
        if (this.map.has(key)) {
            throw new Error("Key already exists");
        }
        else if (!this.contains(item)) {
            return false;
        }
        var added = item.value.pushFront(key);
        if (added !== value_1.NO_VALUE) {
            this.map.set(key, item);
            return true;
        }
        return false;
    };
    /**
     * Adds a key for the supplied item to the end of its queue.
     * @param key
     * @param listNode
     *
     */
    MultiKeyQueue.prototype.addKeyBack = function (key, item) {
        if (this.map.has(key)) {
            throw new Error("Key already exists");
        }
        else if (!this.contains(item)) {
            return false;
        }
        var added = item.value.pushBack(key);
        if (added !== value_1.NO_VALUE) {
            this.map.set(key, item);
            return true;
        }
        return false;
    };
    /**
     * Takes a value from the beginning of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     */
    MultiKeyQueue.prototype.takeFront = function () {
        var item = this.list.takeFront();
        if (item != null) {
            this.dissocKeys(item);
        }
        return item == null ? value_1.NO_VALUE : item;
    };
    /**
     * Takes a value from the end of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     */
    MultiKeyQueue.prototype.takeBack = function () {
        var item = this.list.takeBack();
        if (item != null) {
            this.dissocKeys(item);
        }
        return item == null ? value_1.NO_VALUE : item;
    };
    /**
     * Peeks a value from the beginning of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     *
     */
    MultiKeyQueue.prototype.peekFront = function () {
        var item = this.list.peekFront();
        return item == null ? value_1.NO_VALUE : item;
    };
    /**
     * Peeks a value from the end of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     *
     */
    MultiKeyQueue.prototype.peekBack = function () {
        var item = this.list.peekBack();
        return item == null ? value_1.NO_VALUE : item;
    };
    /**
     * Peeks an item from the beginning of the collection.
     * Returns either item or `NO_VALUE` if queue is empty.
     */
    MultiKeyQueue.prototype.peekItemFront = function () {
        var item = this.list.peekItemFront();
        return item == null ? value_1.NO_VALUE : item;
    };
    /**
     * Peeks an item from the end of the collection.
     * Returns either item or `NO_VALUE` if queue is empty.
     */
    MultiKeyQueue.prototype.peekItemBack = function () {
        var item = this.list.peekItemBack();
        return item == null ? value_1.NO_VALUE : item;
    };
    /**
     * Peeks a key of the item from the beginning of the queue.
     * Returns either key or `NO_VALUE` if queue is empty.
     *
     */
    MultiKeyQueue.prototype.peekKeyFront = function () {
        var first = this.list.peekFront();
        if (first != null) {
            return first.peekFront();
        }
        else {
            return value_1.NO_VALUE;
        }
    };
    /**
     * Peeks a key of the item from the end of the queue.
     * Returns either key or `NO_VALUE` if queue is empty.
     *
     */
    MultiKeyQueue.prototype.peekKeyBack = function () {
        var first = this.list.peekBack();
        if (first != null) {
            return first.peekBack();
        }
        else {
            return value_1.NO_VALUE;
        }
    };
    /**
     * Takes a key of the item from the beginning of the queue.
     * Returns either key or `NO_VALUE` if queue is empty.
     *
     */
    MultiKeyQueue.prototype.takeKeyFront = function () {
        var first = this.list.peekFront();
        if (first != null) {
            var key = first.takeFront();
            if (first.isEmpty()) {
                this.list.takeFront();
            }
            if (key !== value_1.NO_VALUE) {
                this.map.drop(key);
                return key;
            }
        }
        return value_1.NO_VALUE;
    };
    /**
     * Takes a key of the item from the end of the queue.
     * Returns either key or `NO_VALUE` if queue is empty.
     *
     */
    MultiKeyQueue.prototype.takeKeyBack = function () {
        var last = this.list.peekBack();
        if (last != null) {
            var key = last.takeBack();
            if (last.isEmpty()) {
                this.list.takeBack();
            }
            if (key !== value_1.NO_VALUE) {
                this.map.drop(key);
                return key;
            }
        }
        return value_1.NO_VALUE;
    };
    /**
     * Returns an iterator over values.
     *
     */
    MultiKeyQueue.prototype.valuesFront = function () {
        return this.list.valuesFront();
    };
    /**
     * Returns an iterator over values.
     *
     */
    MultiKeyQueue.prototype.valuesBack = function () {
        return this.list.valuesBack();
    };
    /**
     * Returns an iterator over keys starting from the beginning.
     *
     */
    MultiKeyQueue.prototype.keysFront = function () {
        return (0, iterators_1.flatMap)(function (iter) { return iter.keys(); }, this.list.valuesFront());
    };
    /**
     * Returns an iterator over keys starting from the end.
     *
     */
    MultiKeyQueue.prototype.keysBack = function () {
        return (0, iterators_1.flatMap)(function (iter) { return __spreadArray([], __read(iter.keys()), false).reverse(); }, this.list.valuesBack());
    };
    /**
     * Pushes an item to the end of the queue.
     * @param value
     *
     */
    MultiKeyQueue.prototype.pushBack = function (value) {
        var listNode = this.list.pushBack(value);
        this.assocKeys(value, listNode);
        return listNode;
    };
    /**
     * Pushes an item to the beginning of the queue.
     * @param value
     *
     */
    MultiKeyQueue.prototype.pushFront = function (value) {
        var listNode = this.list.pushFront(value);
        this.assocKeys(value, listNode);
        return listNode;
    };
    /**
     * Associates keys of the provided value with the supplied raw linked list node.
     * @param value
     * @param node
     *
     */
    MultiKeyQueue.prototype.assocKeys = function (value, node) {
        var _this = this;
        (0, iterators_1.forEach)(function (key) {
            if (_this.map.has(key)) {
                throw new Error("Key already exists");
            }
            _this.map.set(key, node);
        }, value.keys());
    };
    /**
     * Dissociates keys of the provided value.
     * @param value
     * @param node
     *
     */
    MultiKeyQueue.prototype.dissocKeys = function (value) {
        var _this = this;
        (0, iterators_1.forEach)(function (key) {
            if (!_this.map.drop(key)) {
                throw new Error("Key doesn't exist");
            }
        }, value.keys());
    };
    return MultiKeyQueue;
}(types_1.IndexedOrderedCollectionWithOrderedKeys));
exports.MultiKeyQueue = MultiKeyQueue;
//# sourceMappingURL=MultiKeyQueue.js.map