"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.SingleKeyQueue = void 0;
var MultiKeyQueue_1 = require("../collections/MultiKeyQueue");
var Single_1 = require("../collections/Single");
var types_1 = require("../collections/types");
var iterators_1 = require("../iterators");
/**
 * An indexed queue of items where each item has a single key.
 */
var SingleKeyQueue = /** @class */ (function (_super) {
    __extends(SingleKeyQueue, _super);
    function SingleKeyQueue(values) {
        if (values === void 0) { values = iterators_1.EMPTY_ITERABLE; }
        var _this = _super.call(this) || this;
        _this.inner = new MultiKeyQueue_1.MultiKeyQueue((0, iterators_1.map)(function (value) { return new Single_1.Single(value); }, values));
        return _this;
    }
    /**
     * Returns amount of keys (references) stored in a map.
     *
     */
    SingleKeyQueue.prototype.len = function () {
        return this.inner.len();
    };
    /**
     * Drops an item associated with the supplied key.
     * Returns dropped value in case of a successful drop or `NO_VALUE` if the value wasn't found.
     * @param key
     *
     */
    SingleKeyQueue.prototype.drop = function (key) {
        return this.inner.drop(key);
    };
    /**
     * Moves node to the front of the queue.
     * Returns `true` in case of success.
     * @param item
     *
     */
    SingleKeyQueue.prototype.moveFront = function (item) {
        return this.inner.moveFront(item);
    };
    /**
     * Moves node to the back of the queue.
     * Returns `true` in case of success.
     * @param item
     *
     */
    SingleKeyQueue.prototype.moveBack = function (item) {
        return this.inner.moveBack(item);
    };
    /**
     * Removes an item from the queue.
     * Returns `true` in case of success.
     */
    SingleKeyQueue.prototype.remove = function (item) {
        return this.inner.remove(item);
    };
    /**
     * Returns `true` if supplied item belongs to the collection.
     */
    SingleKeyQueue.prototype.contains = function (item) {
        return this.inner.contains(item);
    };
    /**
     * Drops supplied key from the map.
     * Item belonging to this key will be deleted only if it has no more references in the map.
     * Returns dropped value in case of a successful drop or `NO_VALUE` if the value wasn't found.
     * @param key
     *
     */
    SingleKeyQueue.prototype.dropKey = function (key) {
        return this.inner.dropKey(key);
    };
    /**
     * Inserts given value after the supplied raw linked list node.
     * @param node
     * @param key
     *
     */
    SingleKeyQueue.prototype.insertAfter = function (node, value) {
        return this.inner.insertAfter(node, new Single_1.Single(value));
    };
    /**
     * Inserts given value before the supplied raw linked list node.
     * @param node
     * @param key
     *
     */
    SingleKeyQueue.prototype.insertBefore = function (node, value) {
        return this.inner.insertBefore(node, new Single_1.Single(value));
    };
    /**
     * Returns value associated with the given key or `NO_VALUE` if the value wasn't found.
     * @param key
     *
     */
    SingleKeyQueue.prototype.get = function (key) {
        return this.inner.get(key);
    };
    /**
     * Returns `true` if value associated with the given key exists.
     * @param key
     *
     */
    SingleKeyQueue.prototype.has = function (key) {
        return this.inner.has(key);
    };
    /**
     * Adds a key for the supplied item to the beginning of its queue.
     * @param key
     * @param item
     *
     */
    SingleKeyQueue.prototype.addKeyFront = function (key, item) {
        return this.inner.addKeyFront(key, item);
    };
    /**
     * Adds a key for the supplied item to the end of its queue.
     * @param key
     * @param item
     *
     */
    SingleKeyQueue.prototype.addKeyBack = function (key, item) {
        return this.inner.addKeyBack(key, item);
    };
    /**
     * Takes a value from the beginning of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     */
    SingleKeyQueue.prototype.takeFront = function () {
        return this.inner.takeKeyFront();
    };
    /**
     * Takes a value from the end of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     */
    SingleKeyQueue.prototype.takeBack = function () {
        return this.inner.takeKeyBack();
    };
    /**
     * Peeks a value from the beginning of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     *
     */
    SingleKeyQueue.prototype.peekFront = function () {
        return this.inner.peekKeyFront();
    };
    /**
     * Peeks a value from the end of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     *
     */
    SingleKeyQueue.prototype.peekBack = function () {
        return this.inner.peekKeyBack();
    };
    /**
     * Peeks an item from the beginning of the collection.
     * Returns either item or `NO_VALUE` if queue is empty.
     */
    SingleKeyQueue.prototype.peekItemFront = function () {
        return this.inner.peekItemFront();
    };
    /**
     * Peeks an item from the end of the collection.
     * Returns either item or `NO_VALUE` if queue is empty.
     */
    SingleKeyQueue.prototype.peekItemBack = function () {
        return this.inner.peekItemBack();
    };
    /**
     * Peeks a key of the item from the beginning of the queue.
     * Returns either key or `NO_VALUE` if queue is empty.
     *
     */
    SingleKeyQueue.prototype.peekKeyFront = function () {
        return this.inner.peekKeyFront();
    };
    /**
     * Peeks a key of the item from the end of the queue.
     * Returns either key or `NO_VALUE` if queue is empty.
     *
     */
    SingleKeyQueue.prototype.peekKeyBack = function () {
        return this.inner.peekKeyBack();
    };
    /**
     * Takes a key of the item from the beginning of the queue.
     * Returns either key or `NO_VALUE` if queue is empty.
     *
     */
    SingleKeyQueue.prototype.takeKeyFront = function () {
        return this.inner.takeKeyFront();
    };
    /**
     * Takes a key of the item from the end of the queue.
     * Returns either key or `NO_VALUE` if queue is empty.
     *
     */
    SingleKeyQueue.prototype.takeKeyBack = function () {
        return this.inner.takeKeyBack();
    };
    /**
     * Returns an iterator over values.
     *
     */
    SingleKeyQueue.prototype.valuesFront = function () {
        return this.inner.keysFront();
    };
    /**
     * Returns an iterator over values.
     *
     */
    SingleKeyQueue.prototype.valuesBack = function () {
        return this.inner.keysBack();
    };
    /**
     * Returns an iterator over keys starting from the beginning.
     *
     */
    SingleKeyQueue.prototype.keysFront = function () {
        return this.inner.keysFront();
    };
    /**
     * Returns an iterator over keys starting from the end.
     *
     */
    SingleKeyQueue.prototype.keysBack = function () {
        return this.inner.keysBack();
    };
    /**
     * Pushes an item to the end of the queue.
     * @param value
     *
     */
    SingleKeyQueue.prototype.pushBack = function (value) {
        return this.inner.pushBack(new Single_1.Single(value));
    };
    /**
     * Pushes an item to the beginning of the queue.
     * @param value
     *
     */
    SingleKeyQueue.prototype.pushFront = function (value) {
        return this.inner.pushFront(new Single_1.Single(value));
    };
    return SingleKeyQueue;
}(types_1.IndexedOrderedCollectionWithOrderedKeys));
exports.SingleKeyQueue = SingleKeyQueue;
//# sourceMappingURL=SingleKeyQueue.js.map