"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.LeafStorage = void 0;
var value_1 = require("../value");
var Storage_1 = require("../base/Storage");
/**
 * Stores leaf key -> value pairs.
 */
var LeafStorage = /** @class */ (function (_super) {
    __extends(LeafStorage, _super);
    function LeafStorage(storage, strategy, params, parentPaths) {
        var _this = _super.call(this, params, parentPaths) || this;
        _this.storage = storage;
        _this.strategy = strategy;
        _this.dropStorageValue = _this.storage.drop.bind(_this.storage);
        return _this;
    }
    /**
     * Returns amount of keys (references) stored in a map.
     *
     */
    LeafStorage.prototype.len = function () {
        return this.storage.len();
    };
    /**
     * Drops an item corresponding to the supplied key.
     * @param key
     *
     */
    LeafStorage.prototype.drop = function (key) {
        var isWeak = this.storage.isWeak(key);
        if (!isWeak) {
            var dropped = this.strategy.drop(key);
            if (dropped && this.params.onRemoveLeaf != null)
                this.params.onRemoveLeaf(key);
        }
        var removedFromStorage = this.storage.drop(key);
        if (isWeak) {
            if (removedFromStorage !== value_1.NO_VALUE && this.params.onRemoveLeaf != null)
                this.params.onRemoveLeaf(key);
        }
        return removedFromStorage;
    };
    /**
     * Retrieves an item corresponding to the supplied key.
     * @param key
     *
     */
    LeafStorage.prototype.get = function (key) {
        var isWeak = this.storage.isWeak(key);
        var extracted = this.storage.get(key);
        if (!isWeak && extracted !== value_1.NO_VALUE) {
            this.handleResult(this.strategy.read(key));
        }
        return extracted;
    };
    /**
     * Associates supplied item with the key.
     * @param key
     * @param value
     *
     */
    LeafStorage.prototype.set = function (key, value) {
        var isWeak = this.storage.isWeak(key);
        if (!isWeak)
            this.handleResult(this.strategy.write(key));
        this.storage.set(key, value);
    };
    /**
     * Removes an item from the beginning of the queue.
     *
     */
    LeafStorage.prototype.take = function () {
        var key = this.strategy.take();
        if (key !== value_1.NO_VALUE) {
            this.storage.drop(key);
            if (this.params.onRemoveLeaf != null)
                this.params.onRemoveLeaf(key);
        }
        return key;
    };
    /**
     * Removes all items from the storage and cache strategy.
     *
     */
    LeafStorage.prototype.clear = function () {
        this.storage.clear();
        this.strategy.clear();
    };
    /**
     * Returns an iterator over the entries.
     *
     */
    LeafStorage.prototype.entries = function () {
        return this.storage.entries();
    };
    /**
     * Executes callbacks for each added/removed item.
     * @param result
     */
    LeafStorage.prototype.handleResult = function (result) {
        result.forEachRemoved(this.dropStorageValue);
        if (this.params.onCreateLeaf != null) {
            result.forEachAdded(this.params.onCreateLeaf);
        }
        if (this.params.onRemoveLeaf != null) {
            result.forEachRemoved(this.params.onRemoveLeaf);
        }
    };
    return LeafStorage;
}(Storage_1.Storage));
exports.LeafStorage = LeafStorage;
//# sourceMappingURL=LeafStorage.js.map