"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.RootLeafStrategy = void 0;
var CacheStrategy_1 = require("../base/CacheStrategy");
var iterators_1 = require("../iterators");
var utils_1 = require("../utils");
var value_1 = require("../value");
/**
 * Cache strategy for leaf values.
 */
var RootLeafStrategy = /** @class */ (function (_super) {
    __extends(RootLeafStrategy, _super);
    function RootLeafStrategy(leafCapacity, strategy) {
        var _this = _super.call(this, leafCapacity) || this;
        _this._leafStorageStrategy = strategy;
        _this._leaves = 0;
        return _this;
    }
    /**
     * Records a read access of the supplied value.
     */
    RootLeafStrategy.prototype.read = function (value) {
        return this.handleInnerStrategyResult(this._leafStorageStrategy.read(value));
    };
    /**
     * Records a write access of the supplied value.
     */
    RootLeafStrategy.prototype.write = function (value) {
        var res = !this.has(value)
            ? this.reservePlace()
            : utils_1.Result.empty();
        this._leaves++;
        return res.chain(this.handleInnerStrategyResult(this._leafStorageStrategy.write(value)));
    };
    /**
     * Removes supplied value from the cache.
     */
    RootLeafStrategy.prototype.drop = function (value) {
        var dropped = this._leafStorageStrategy.drop(value);
        if (dropped) {
            this._leaves -= value.len();
        }
        return dropped;
    };
    /**
     * Returns underlying amount of tracked cache entries.
     */
    RootLeafStrategy.prototype.len = function () {
        return this._leaves;
    };
    /**
     * Returns `true` if supplied node exists in cache.
     */
    RootLeafStrategy.prototype.has = function (node) {
        return this._leafStorageStrategy.has(node);
    };
    /**
     * Peeks an item from the beginning of the queue.
     */
    RootLeafStrategy.prototype.peek = function () {
        return this._leafStorageStrategy.peek();
    };
    /**
     * Removes an item from the beginning of the queue.
     *
     */
    RootLeafStrategy.prototype.take = function () {
        var maybeStorage = this.peek();
        if (maybeStorage !== value_1.NO_VALUE) {
            var storage = maybeStorage;
            var removed = storage.take();
            if (removed !== value_1.NO_VALUE) {
                this._leaves--;
            }
            if (storage.isEmpty()) {
                this.drop(storage);
                return storage;
            }
        }
        return value_1.NO_VALUE;
    };
    /**
     * Handles a result produced by calling one of the inner strategy methods.
     */
    RootLeafStrategy.prototype.handleInnerStrategyResult = function (result) {
        var _this = this;
        (0, iterators_1.forEach)(function (removed) { return (_this._leaves -= removed.len()); }, result.removed);
        return result;
    };
    return RootLeafStrategy;
}(CacheStrategy_1.CacheStrategy));
exports.RootLeafStrategy = RootLeafStrategy;
//# sourceMappingURL=RootLeafStrategy.js.map