"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.ObjectStorage = void 0;
var value_1 = require("../value");
var Storage_1 = require("../base/Storage");
var iterators_1 = require("../iterators");
/**
 * Key-value storage for values with primitive keys based on an `Object`.
 */
var ObjectStorage = /** @class */ (function (_super) {
    __extends(ObjectStorage, _super);
    function ObjectStorage(params, rootPath) {
        var _this = _super.call(this, params, rootPath) || this;
        _this.length = 0;
        _this.map = Object.create(null);
        return _this;
    }
    /**
     * Returns amount of items stored in a map.
     *
     */
    ObjectStorage.prototype.len = function () {
        return this.length;
    };
    /**
     * Returns `true` if value associated with the given key exists.
     * @param rawKey
     *
     */
    ObjectStorage.prototype.has = function (rawKey) {
        var objKey = this.objectKeyFromRawKey(rawKey);
        return objKey in this.map;
    };
    /**
     * Retrieves an item corresponding to the supplied key.
     * @param rawKey
     *
     */
    ObjectStorage.prototype.get = function (rawKey) {
        var objKey = this.objectKeyFromRawKey(rawKey);
        if (objKey in this.map) {
            return this.map[objKey];
        }
        else {
            return value_1.NO_VALUE;
        }
    };
    /**
     * Drops an item corresponding to the supplied key.
     * @param key
     *
     */
    ObjectStorage.prototype.drop = function (rawKey) {
        var objKey = this.objectKeyFromRawKey(rawKey);
        var value = value_1.NO_VALUE;
        if (objKey in this.map) {
            this.length--;
            value = this.map[objKey];
            delete this.map[objKey];
        }
        return value;
    };
    /**
     * Associates supplied item with the key.
     * @param rawKey
     * @param value
     *
     */
    ObjectStorage.prototype.set = function (rawKey, value) {
        var objKey = this.objectKeyFromRawKey(rawKey);
        var has = objKey in this.map;
        if (!has) {
            this.length++;
            this.map[objKey] = value;
        }
    };
    /**
     * Removes all items from the storage.
     *
     */
    ObjectStorage.prototype.clear = function () {
        this.length = 0;
        this.map = Object.create(null);
    };
    /**
     * Returns an iterator over the entries.
     *
     */
    ObjectStorage.prototype.entries = function () {
        var _this = this;
        return (0, iterators_1.map)(function (key) { return ({
            key: _this.rawKeyFromObjectKey(key),
            value: _this.map[key]
        }); }, Reflect.ownKeys(this.map));
    };
    /**
     * Converts provided raw key to the object key.
     * @param key
     */
    ObjectStorage.prototype.objectKeyFromRawKey = function (key) {
        var type = typeof key;
        switch (type) {
            case "number":
                return "0".concat(key);
            case "string":
                return "1".concat(key);
            case "boolean":
                return "2".concat(key);
            case "bigint":
                return "3".concat(key);
            case "undefined":
                return "4";
            case "symbol":
                return key;
            default:
                if (key === null)
                    return "5";
                throw new TypeError("Invalid value ".concat(String(key), " with type ").concat(type, ", expected primitive"));
        }
    };
    /**
     * Instantiates a raw key from the provided object key.
     */
    ObjectStorage.prototype.rawKeyFromObjectKey = function (key) {
        switch (typeof key) {
            case "symbol":
                return key;
            case "string":
                var type = +key[0];
                switch (type) {
                    case 0:
                        return Number(key.slice(1));
                    case 1:
                        return key.slice(1);
                    case 2:
                        return (key.slice(1) === "true");
                    case 3:
                        return BigInt(key.slice(1));
                    case 4:
                        return undefined;
                    case 5:
                        return null;
                    default:
                        throw new TypeError("Invalid type ".concat(type, " of key ").concat(key));
                }
            default:
                throw new TypeError("Invalid key ".concat(key, ", expected string or symbol"));
        }
    };
    return ObjectStorage;
}(Storage_1.Storage));
exports.ObjectStorage = ObjectStorage;
//# sourceMappingURL=ObjectStorage.js.map