"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.UnifiedStorage = void 0;
var value_1 = require("../value");
var Storage_1 = require("../base/Storage");
var WeakStorage_1 = require("./WeakStorage");
var ObjectStorage_1 = require("./ObjectStorage");
var MapStorage_1 = require("./MapStorage");
var iterators_1 = require("../iterators");
var utils_1 = require("../utils");
/**
 * Storage for values with both primitive and non-primitive keys.
 */
var UnifiedStorage = /** @class */ (function (_super) {
    __extends(UnifiedStorage, _super);
    function UnifiedStorage(params, rootPath) {
        var _this = _super.call(this, params, rootPath) || this;
        var isMapImplemented = (0, utils_1.mapImplemented)();
        var NonPrimitiveStorage = (params === null || params === void 0 ? void 0 : params.useWeakStorage) || !isMapImplemented
            ? WeakStorage_1.WeakStorage
            : MapStorage_1.MapStorage;
        var PrimitiveStorage = (params === null || params === void 0 ? void 0 : params.useObjectStorage) || !isMapImplemented
            ? ObjectStorage_1.ObjectStorage
            : MapStorage_1.MapStorage;
        _this.nonPrimitiveStorage = new NonPrimitiveStorage();
        _this.primitiveStorage = new PrimitiveStorage();
        return _this;
    }
    /**
     * Returns `true` if supplied is weak, and thus won't be stored directly.
     * @param key
     */
    UnifiedStorage.prototype.isWeak = function (key) {
        return !(0, value_1.isPrimitiveValue)(key);
    };
    /**
     * Returns amount of items stored in a map.
     *
     */
    UnifiedStorage.prototype.len = function () {
        return this.primitiveStorage.len() + this.nonPrimitiveStorage.len();
    };
    /**
     * Returns `true` if value associated with the given key exists.
     * @param key
     *
     */
    UnifiedStorage.prototype.has = function (key) {
        return (0, value_1.isPrimitiveValue)(key)
            ? this.primitiveStorage.has(key)
            : this.nonPrimitiveStorage.has(key);
    };
    /**
     * Retrieves an item corresponding to the supplied key.
     * @param key
     *
     */
    UnifiedStorage.prototype.get = function (key) {
        return (0, value_1.isPrimitiveValue)(key)
            ? this.primitiveStorage.get(key)
            : this.nonPrimitiveStorage.get(key);
    };
    /**
     * Drops an item corresponding to the supplied key.
     * @param key
     *
     */
    UnifiedStorage.prototype.drop = function (key) {
        return (0, value_1.isPrimitiveValue)(key)
            ? this.primitiveStorage.drop(key)
            : this.nonPrimitiveStorage.drop(key);
    };
    /**
     * Associates supplied item with the key.
     * @param key
     * @param value
     *
     */
    UnifiedStorage.prototype.set = function (key, value) {
        if ((0, value_1.isPrimitiveValue)(key)) {
            this.primitiveStorage.set(key, value);
        }
        else {
            this.nonPrimitiveStorage.set(key, value);
        }
    };
    /**
     * Removes all items from the storage.
     *
     */
    UnifiedStorage.prototype.clear = function () {
        this.primitiveStorage.clear();
        this.nonPrimitiveStorage.clear();
    };
    /**
     * Returns an iterator over the entries.
     *
     */
    UnifiedStorage.prototype.entries = function () {
        return (0, iterators_1.chain)(this.primitiveStorage.entries(), this.nonPrimitiveStorage.entries());
    };
    return UnifiedStorage;
}(Storage_1.Storage));
exports.UnifiedStorage = UnifiedStorage;
//# sourceMappingURL=UnifiedStorage.js.map