"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.WeakStorage = void 0;
require("weakmap-polyfill");
var Storage_1 = require("../base/Storage");
var value_1 = require("../value");
var iterators_1 = require("../iterators");
/**
 * Weak storage for values with non-primitive keys.
 */
var WeakStorage = /** @class */ (function (_super) {
    __extends(WeakStorage, _super);
    function WeakStorage(params, rootPath) {
        var _this = _super.call(this, params, rootPath) || this;
        _this.weakMap = new WeakMap();
        _this.addedEntries = 0;
        return _this;
    }
    /**
     * Returns `true` if supplied is weak, and thus won't be stored directly.
     * @param key
     */
    WeakStorage.prototype.isWeak = function (_key) {
        return true;
    };
    /**
     * Returns amount of items added to the storage.
     *
     */
    WeakStorage.prototype.len = function () {
        return this.addedEntries;
    };
    /**
     * Returns `true` if value associated with the given key exists.
     * @param key
     *
     */
    WeakStorage.prototype.has = function (key) {
        return this.weakMap.has(key);
    };
    /**
     * Retrieves an item corresponding to the supplied key.
     * @param key
     *
     */
    WeakStorage.prototype.get = function (key) {
        if (this.weakMap.has(key)) {
            return this.weakMap.get(key);
        }
        else {
            return value_1.NO_VALUE;
        }
    };
    /**
     * Drops an item corresponding to the supplied key.
     * @param key
     *
     */
    WeakStorage.prototype.drop = function (key) {
        var value = this.get(key);
        var removed = this.weakMap["delete"](key);
        if (removed) {
            this.addedEntries--;
        }
        return value;
    };
    /**
     * Associates supplied item with the key.
     * @param key
     * @param value
     *
     */
    WeakStorage.prototype.set = function (key, value) {
        if (!this.weakMap.has(key)) {
            this.addedEntries++;
        }
        this.weakMap.set(key, value);
    };
    /**
     * Does nothing.
     *
     */
    WeakStorage.prototype.clear = function () { };
    /**
     * Returns an iterator over the entries.
     *
     */
    WeakStorage.prototype.entries = function () {
        return (0, iterators_1.empty)();
    };
    return WeakStorage;
}(Storage_1.Storage));
exports.WeakStorage = WeakStorage;
//# sourceMappingURL=WeakStorage.js.map