"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
exports.__esModule = true;
exports.LFU = void 0;
var CacheStrategy_1 = require("../base/CacheStrategy");
var collections_1 = require("../collections");
var value_1 = require("../value");
var utils_1 = require("../utils");
var iterators_1 = require("../iterators");
/**
 * `L`east `F`requently `U`used cache replacement policy.
 */
var LFU = /** @class */ (function (_super) {
    __extends(LFU, _super);
    function LFU(capacity) {
        var _this = _super.call(this, capacity) || this;
        _this.queue = new collections_1.MultiKeyQueue();
        return _this;
    }
    /**
     * Returns amount of keys (references) stored in a map.
     *
     */
    LFU.prototype.len = function () {
        return this.queue.len();
    };
    /**
     * Records read access of the supplied item.
     * @param value
     *
     */
    LFU.prototype.write = function (value) {
        if (!this.has(value)) {
            var res = this.reservePlace();
            var head = this.queue.peekItemFront();
            if (head !== value_1.NO_VALUE) {
                var isFirstLevel = head.value.level === 1;
                if (isFirstLevel) {
                    var addedHead = this.queue.addKeyBack(value, head);
                    if (!addedHead) {
                        throw new Error("`LFU`: failed to modify cache head");
                    }
                    return res.chain(utils_1.Result.added((0, iterators_1.once)(value)));
                }
            }
            var pushed = this.queue.pushFront(new LevelEntry(1, value));
            if (pushed === value_1.NO_VALUE) {
                throw new Error("`LFU`: failed to create the cache head");
            }
            return res.chain(utils_1.Result.added((0, iterators_1.once)(value)));
        }
        else {
            return this.read(value);
        }
    };
    /**
     * Records write access of the supplied item.
     * @param value
     *
     */
    LFU.prototype.read = function (value) {
        var maybeNoValueListNode = this.queue.get(value);
        if (maybeNoValueListNode === value_1.NO_VALUE) {
            throw new Error("`LFU`: cache entry doesn't exist");
        }
        var listNode = maybeNoValueListNode;
        var next = listNode.next;
        if (!this.queue.dropKey(value)) {
            throw new Error("`LFU`: failed to drop a key in the current cache level");
        }
        var newLevel = listNode.value.level + 1;
        if (next) {
            if (next.value.level === newLevel) {
                var added = this.queue.addKeyBack(value, next);
                if (!added) {
                    throw new Error("`LFU`: failed to move cache value to the next cache level");
                }
            }
            else {
                var inserted = this.queue.insertBefore(next, new LevelEntry(newLevel, value));
                if (inserted === value_1.NO_VALUE) {
                    throw new Error("`LFU`: failed to insert a new level of cache");
                }
            }
        }
        else {
            var pushed = this.queue.pushBack(new LevelEntry(newLevel, value));
            if (pushed === value_1.NO_VALUE) {
                throw new Error("`LFU`: failed to push a new cache level");
            }
        }
        return utils_1.Result.empty();
    };
    /**
     * Returns `true` if given item exists in the queue.
     * @param value
     *
     */
    LFU.prototype.has = function (value) {
        return this.queue.has(value);
    };
    /**
     * Removes supplied item from the queue.
     * @param value
     *
     */
    LFU.prototype.drop = function (value) {
        return this.queue.dropKey(value);
    };
    /**
     * Takes a value from the beginning of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     *
     */
    LFU.prototype.take = function () {
        return this.queue.takeKeyFront();
    };
    /**
     * Peeks a value from the beginning of the queue.
     * Returns either item or `NO_VALUE` if queue is empty.
     *
     */
    LFU.prototype.peek = function () {
        return this.queue.peekKeyFront();
    };
    return LFU;
}(CacheStrategy_1.CacheStrategy));
exports.LFU = LFU;
/** Describes a cache entry containing ordered values and its level. */
var LevelEntry = /** @class */ (function (_super) {
    __extends(LevelEntry, _super);
    function LevelEntry(level, value) {
        var _this = _super.call(this, (0, iterators_1.once)(value)) || this;
        _this.level = level;
        return _this;
    }
    return LevelEntry;
}(collections_1.SingleKeyQueue));
//# sourceMappingURL=lfu.js.map